@@ -29,9 +29,9 @@ def addToDescription(rowobj):
 
 def subcomponents(rowobj): #UPDATE TO WORK WITH CELL DICT, ALLOW CONSTRAINTS
 	if 'subcomp' in rowobj.col_cell_dict:
-		subcomps = rowobj.col_cell_dict['subcomp'].values()
+		subcomps = list(rowobj.col_cell_dict['subcomp'].values())
 	if 'constraint' in rowobj.col_cell_dict:
-		constraints = rowobj.col_cell_dict['constraint'].values()
+		constraints = list(rowobj.col_cell_dict['constraint'].values())
 	else:
 		constraints = []
 
@@ -63,8 +63,7 @@ def subcomponents(rowobj): #UPDATE TO WORK WITH CELL DICT, ALLOW CONSTRAINTS
 		template = rowobj.obj_dict[f'{rowobj.obj.displayId}_template']['object']
 
 		for sub in comp_list:
-			name = f'{sbol3.get_namespace()}{sub}'
-			name = helpers.check_name(name)
+			name = f'{sbol3.get_namespace()}{helpers.check_name(sub)}'
 			sub_part = sbol3.SubComponent(name)
 			template.features.append(sub_part)
 		# template.assemblePrimaryStructure(comp_list)
@@ -78,14 +77,14 @@ def subcomponents(rowobj): #UPDATE TO WORK WITH CELL DICT, ALLOW CONSTRAINTS
 			var_comp.variable = variant_comps[var]['object']
 
 			var_list = re.split(",", variant_comps[var]['variant_list'])
-			var_list = [f'{sbol3.get_namespace()}{x.strip()}' for x in var_list]
+			var_list = [f'{sbol3.get_namespace()}{helpers.check_name(x.strip())}' for x in var_list]
 			var_comp.variants = var_list
 			rowobj.obj.variable_features.append(var_comp)
 
 	else:
 		raise KeyError(f'The object type "{type(rowobj.obj)}" does not allow subcomponents. (sheet:{rowobj.sheet}, row:{rowobj.sht_row}, sbol term dict:{rowobj.col_cell_dict})')
 
-def dataSource(rowobj): #UPDATE TO WORK ON MULTI COLUMN??? WITH CELL DICT
+def dataSource(rowobj):
 	prefs = rowobj.col_cell_dict['pref']
 	vals = rowobj.col_cell_dict['val']
 	for colnum in range(len(prefs.keys())):
@@ -148,20 +147,32 @@ def sequence(rowobj):
 		else:
 			raise TypeError(f"A multicolumn value was unexpectedly given in sequence, {rowobj.col_cell_dict}")
 
+def circular(rowobj):
+	# if false add to linear collection if true add to types
+	pass
+
 def finalProduct(rowobj):
 	# create final products collection if it doesn't yet exist
 	# add object to collection
 	columns = rowobj.col_cell_dict
-
+ 
+	print(rowobj.col_cell_dict)
 	for col in columns:
 		# check if the cell value is true
+		print(col)
 		if columns[col]:
 			doc = rowobj.doc
 
 			sbol_objs = doc.objects
 			sbol_objs_names = [x.name for x in sbol_objs]
+			print(sbol_objs_names)
 			if 'FinalProducts' not in sbol_objs:
 				colec = sbol3.Collection('FinalProducts', name='FinalProducts')
+
+				sbol_objs = doc.objects
+				sbol_objs_names = [x.name for x in sbol_objs]
+				print(sbol_objs_names)
+
 				doc.add(colec)
 			else:
 				colec = sbol_objs[sbol_objs_names.index('FinalProducts')]
